@@ -2,21 +2,25 @@
  * @file osso-rpc.c
  * This file implements rpc calls from one application to another.
  * 
- * Copyright (C) 2005-2006 Nokia Corporation.
+ * This file is part of libosso
+ *
+ * Copyright (C) 2005-2006 Nokia Corporation. All rights reserved.
+ *
+ * Contact: Kimmo Hmlinen <kimmo.hamalainen@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation;
- * version 2.1 of the License.
+ * modify it under the terms of the GNU Lesser General Public License
+ * version 2.1 as published by the Free Software Foundation.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301 USA
  */
 
 #define TASK_NAV_SERVICE "com.nokia.tasknav"
@@ -30,11 +34,11 @@
 
 
 #include "osso-internal.h"
+#include <assert.h>
 
-static void _rm_cb_f (osso_context_t * osso, const gchar * interface,
-                      osso_rpc_cb_f * cb, gpointer data);
 static DBusHandlerResult _rpc_handler (osso_context_t * osso,
-                                       DBusMessage * msg, gpointer data);
+                                       DBusMessage * msg,
+                                       _osso_callback_data_t *data);
 static void _append_args(DBusMessage *msg, int type, va_list var_args);
 static void _append_arg (DBusMessage * msg, osso_rpc_t * arg);
 static void _get_arg (DBusMessageIter * iter, osso_rpc_t * retval);
@@ -49,12 +53,6 @@ static osso_return_t _rpc_run_with_argfill (osso_context_t *osso,
 					    osso_rpc_argfill *argfill,
 					    void *argfill_data);
 
-typedef struct {
-    osso_rpc_cb_f *func;
-    osso_rpc_retval_free_f *retval_free;
-    gpointer data;
-}_osso_rpc_t;
-
 typedef struct {
     osso_rpc_async_f *func;
     gpointer data;
@@ -96,7 +94,7 @@ osso_return_t osso_rpc_run(osso_context_t *osso, const gchar *service,
     fill_from_va_list_data data;
     
     if( (osso == NULL) || (service == NULL) || (object_path == NULL) ||
-	(interface == NULL) || (method == NULL))
+	(interface == NULL) || (method == NULL) || (osso->conn == NULL))
 	return OSSO_INVALID;
 
     data.argument_type = argument_type;
@@ -122,7 +120,7 @@ osso_return_t osso_rpc_run_with_argfill (osso_context_t *osso,
 					 void *argfill_data)
 {
     if( (osso == NULL) || (service == NULL) || (object_path == NULL) ||
-	(interface == NULL) || (method == NULL))
+	(interface == NULL) || (method == NULL) || (osso->conn == NULL))
 	return OSSO_INVALID;
 
     return _rpc_run_with_argfill (osso, osso->conn,
@@ -142,7 +140,7 @@ osso_return_t osso_rpc_run_system(osso_context_t *osso, const gchar *service,
     fill_from_va_list_data data;
     
     if( (osso == NULL) || (service == NULL) || (object_path == NULL) ||
-	(interface == NULL) || (method == NULL))
+	(interface == NULL) || (method == NULL) || (osso->sys_conn == NULL))
 	return OSSO_INVALID;
 
     data.argument_type = argument_type;
@@ -168,7 +166,7 @@ osso_return_t osso_rpc_run_system_with_argfill (osso_context_t *osso,
 						void *argfill_data)
 {
     if( (osso == NULL) || (service == NULL) || (object_path == NULL) ||
-	(interface == NULL) || (method == NULL))
+	(interface == NULL) || (method == NULL) || (osso->sys_conn == NULL))
 	return OSSO_INVALID;
 
     return _rpc_run_with_argfill (osso, osso->sys_conn,
@@ -316,32 +314,29 @@ osso_return_t osso_rpc_run_with_defaults(osso_context_t *osso,
 					 int argument_type,
 					 ...)
 {
-    char service[MAX_SVC_LEN] = {0}, object_path[MAX_OP_LEN] = {0},
-         interface[MAX_IF_LEN] = {0};
+    char service[MAX_SVC_LEN], path[MAX_OP_LEN], interface[MAX_IF_LEN];
     fill_from_va_list_data data;
     osso_return_t ret;
-    gchar* copy = NULL;
 
-    dprint("");
-    if( (osso == NULL) || (application == NULL) || (method == NULL) )
+    if( (osso == NULL) || (application == NULL) || (method == NULL)
+        || (osso->conn == NULL) )
 	return OSSO_INVALID;
-    
-    g_snprintf(service, MAX_SVC_LEN, OSSO_BUS_ROOT ".%s", application);
-    copy = appname_to_valid_path_component(application);
-    if (copy == NULL) {
-	ULOG_ERR_F("failure in appname_to_valid_path_component");
-	return OSSO_ERROR;
-    }
-    g_snprintf(object_path, MAX_OP_LEN, OSSO_BUS_ROOT_PATH "/%s", copy);
-    g_free(copy); copy = NULL;
-    g_snprintf(interface, MAX_IF_LEN, "%s", service);
 
+    make_default_service(application, service);
+
+    if (!make_default_object_path(application, path)) {
+        ULOG_ERR_F("make_default_object_path() failed");
+        return OSSO_ERROR;
+    }
+
+    make_default_interface(application, interface);
+    
     data.argument_type = argument_type;
     va_start(data.arg_list, argument_type);
 
-    ret = _rpc_run_with_argfill (osso, osso->conn,
-				 service, object_path, interface, method,
-				 retval,
+    ret = _rpc_run_with_argfill (osso, osso->conn, service,
+                                 path, interface,
+                                 method, retval,
 				 fill_from_va_list, &data);
     
     va_end(data.arg_list);
@@ -349,6 +344,21 @@ osso_return_t osso_rpc_run_with_defaults(osso_context_t *osso,
     return ret;
 }
 
+static void free_osso_rpc_async_t(_osso_rpc_async_t *rpc)
+{
+    if (rpc != NULL) {
+        if (rpc->interface != NULL) {
+            g_free(rpc->interface);
+            rpc->interface = NULL;
+        }
+        if (rpc->method != NULL) {
+            g_free(rpc->method);
+            rpc->method = NULL;
+        }
+        free(rpc); /* free() because calloc() was used */
+    }
+}
+
 /************************************************************************/
 osso_return_t osso_rpc_async_run_with_argfill (osso_context_t *osso,
 						  const gchar *service,
@@ -417,6 +427,10 @@ osso_return_t osso_rpc_async_run_with_argfill (osso_context_t *osso,
 					 _async_return_handler,
 					 rpc, NULL);
 	}
+        else if (rpc != NULL) {
+            free_osso_rpc_async_t(rpc);
+            rpc = NULL;
+        }
 	dbus_message_unref(msg);
 
 	/* Tell TaskNavigator to show "launch banner" */
@@ -445,6 +459,10 @@ osso_return_t osso_rpc_async_run_with_argfill (osso_context_t *osso,
     else {
         ULOG_ERR_F("dbus_connection_send(_with_reply) failed");
         dbus_message_unref(msg);
+        if (rpc != NULL) {
+            free_osso_rpc_async_t(rpc);
+            rpc = NULL;
+        }
         return OSSO_ERROR;
     }
 }
@@ -489,32 +507,29 @@ osso_return_t osso_rpc_async_run_with_defaults(osso_context_t *osso,
 					       gpointer data,
 					       int argument_type, ...)
 {
+    char service[MAX_SVC_LEN], path[MAX_OP_LEN], interface[MAX_IF_LEN];
     fill_from_va_list_data argfill_data;
     osso_return_t ret;
-    char service[MAX_SVC_LEN] = {0}, object_path[MAX_OP_LEN] = {0},
-         interface[MAX_IF_LEN] = {0};
-    gchar* copy = NULL;
 
     if( (osso == NULL) || (application == NULL) || (method == NULL) )
 	return OSSO_INVALID;
-    
-    g_snprintf(service, MAX_SVC_LEN, OSSO_BUS_ROOT ".%s", application);
-    copy = appname_to_valid_path_component(application);
-    if (copy == NULL) {
-        ULOG_ERR_F("failure in appname_to_valid_path_component");
-	return OSSO_ERROR;
-    }
-    g_snprintf(object_path, MAX_OP_LEN, OSSO_BUS_ROOT_PATH "/%s", copy);
-    g_free(copy); copy = NULL;
-    g_snprintf(interface, MAX_IF_LEN, "%s", service);
 
+    make_default_service(application, service);
+
+    if (!make_default_object_path(application, path)) {
+        ULOG_ERR_F("make_default_object_path() failed");
+        return OSSO_ERROR;
+    }
+
+    make_default_interface(application, interface);
+    
     argfill_data.argument_type = argument_type;
     va_start(argfill_data.arg_list, argument_type);
 
-    ret = osso_rpc_async_run_with_argfill (osso,
-				       service, object_path, interface, method,
-				       async_cb, data,
-				       fill_from_va_list, &argfill_data);
+    ret = osso_rpc_async_run_with_argfill (osso, service, path,
+                                           interface, method,
+				           async_cb, data,
+				           fill_from_va_list, &argfill_data);
     dprint("osso_rpc_async_run_with_argfill returned");
     
     va_end(argfill_data.arg_list);
@@ -531,64 +546,67 @@ static osso_return_t _rpc_set_cb_f(osso_context_t *osso,
                                    osso_rpc_retval_free_f *retval_free,
                                    gboolean use_system_bus)
 {
-    DBusError err;
-    struct DBusObjectPathVTable vt;
-    _osso_rpc_t *rpc;
+    _osso_callback_data_t *rpc;
     int ret;
-    dbus_bool_t bret;
 
     ULOG_DEBUG_F("s '%s' o '%s' i '%s', %s bus",
                  service, object_path, interface,
                  use_system_bus ? "system" : "session");
     
-    rpc = (_osso_rpc_t *)calloc(1, sizeof(_osso_rpc_t));
+    rpc = calloc(1, sizeof(_osso_callback_data_t));
     if (rpc == NULL) {
         ULOG_ERR_F("calloc failed");
         return OSSO_ERROR;
     }
     
-    dbus_error_init(&err);
+    if (strcmp(service, osso->service) != 0
+        || (use_system_bus && !osso->systembus_service_registered)
+        || (!use_system_bus && !osso->sessionbus_service_registered)) {
+        DBusError err;
 
-    dprint("requesting service '%s'", service);
-    ret = dbus_bus_request_name(use_system_bus ? osso->sys_conn :
-              osso->conn, service, DBUS_NAME_FLAG_ALLOW_REPLACEMENT |
-              DBUS_NAME_FLAG_REPLACE_EXISTING |
-              DBUS_NAME_FLAG_DO_NOT_QUEUE, &err);
-    if (ret == DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER ||
-        ret == DBUS_REQUEST_NAME_REPLY_ALREADY_OWNER) {
-        /* success */ 
-    } else if (ret == DBUS_REQUEST_NAME_REPLY_IN_QUEUE ||
-               ret == DBUS_REQUEST_NAME_REPLY_EXISTS) {
-        /* this should be impossible */
-        ULOG_ERR_F("dbus_bus_request_name is broken");
-        return OSSO_ERROR;
-    } else if (ret == -1) {
-        ULOG_ERR_F("dbus_bus_request_name for '%s' failed: %s",
-                   service, err.message);
-	dbus_error_free(&err);
-        return OSSO_ERROR;
+        dbus_error_init(&err);
+
+        ULOG_DEBUG_F("requesting service '%s'", service);
+        ret = dbus_bus_request_name(use_system_bus ? osso->sys_conn :
+                  osso->conn, service, DBUS_NAME_FLAG_ALLOW_REPLACEMENT |
+                  DBUS_NAME_FLAG_REPLACE_EXISTING |
+                  DBUS_NAME_FLAG_DO_NOT_QUEUE, &err);
+        if (ret == DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER ||
+            ret == DBUS_REQUEST_NAME_REPLY_ALREADY_OWNER) {
+            /* success */ 
+        } else if (ret == DBUS_REQUEST_NAME_REPLY_IN_QUEUE ||
+                   ret == DBUS_REQUEST_NAME_REPLY_EXISTS) {
+            /* this should be impossible */
+            ULOG_ERR_F("dbus_bus_request_name is broken");
+            free(rpc);
+            return OSSO_ERROR;
+        } else if (ret == -1) {
+            ULOG_ERR_F("dbus_bus_request_name for '%s' failed: %s",
+                       service, err.message);
+            dbus_error_free(&err);
+            free(rpc);
+            return OSSO_ERROR;
+        }
+
+        if (strcmp(service, osso->service) == 0) {
+            if (use_system_bus) {
+                osso->systembus_service_registered = TRUE;
+            } else {
+                osso->sessionbus_service_registered = TRUE;
+            }
+        }
     }
 
-    vt.message_function = _msg_handler;
-    vt.unregister_function = NULL;
-    
-    dprint("registering object_path: '%s'", object_path);
-    bret = dbus_connection_register_object_path(use_system_bus ?
-               osso->sys_conn : osso->conn, object_path, &vt,
-               (void *)osso);
-    if (!bret) {
-        ULOG_WARN_F("ignoring dbus_connection_register_object_path"
-                    " failure, maybe it was already registered");
-	/* error ignored because we cannot distinguish OOM and already
-	 * registered object path */
-    }
+    rpc->user_cb = cb;
+    rpc->user_data = data;
+    rpc->data = retval_free;
 
-    rpc->func = cb;
-    rpc->retval_free = retval_free;
-    rpc->data = data;
-
-    _msg_handler_set_cb_f(osso, interface, _rpc_handler,
-                          (gpointer)rpc, TRUE);
+    _msg_handler_set_cb_f(osso,
+                          service,
+                          object_path,
+                          interface,
+                          _rpc_handler,
+                          rpc, TRUE);
     return OSSO_OK;
 }
 
@@ -604,10 +622,12 @@ osso_return_t _test_rpc_set_cb_f(osso_context_t *osso, const gchar *service,
 }
 
 /************************************************************************/
-osso_return_t osso_rpc_set_cb_f_with_free (osso_context_t *osso, const gchar *service,
-					   const gchar *object_path, const gchar *interface,
-					   osso_rpc_cb_f *cb, gpointer data,
-					   osso_rpc_retval_free_f *retval_free)
+osso_return_t osso_rpc_set_cb_f_with_free (osso_context_t *osso,
+                                           const gchar *service,
+                                           const gchar *object_path,
+                                           const gchar *interface,
+                                           osso_rpc_cb_f *cb, gpointer data,
+                                           osso_rpc_retval_free_f *retval_free)
 {
     if( (osso == NULL) || (service == NULL) || (object_path == NULL) ||
 	(interface == NULL) || (cb == NULL))
@@ -616,11 +636,14 @@ osso_return_t osso_rpc_set_cb_f_with_free (osso_context_t *osso, const gchar *se
 			 interface,cb, data, retval_free, FALSE);
 }
 
-osso_return_t osso_rpc_set_cb_f (osso_context_t *osso, const gchar *service,
-				 const gchar *object_path, const gchar *interface,
-				 osso_rpc_cb_f *cb, gpointer data)
+osso_return_t osso_rpc_set_cb_f (osso_context_t *osso,
+                                 const gchar *service,
+                                 const gchar *object_path,
+                                 const gchar *interface,
+                                 osso_rpc_cb_f *cb, gpointer data)
 {
-    return osso_rpc_set_cb_f_with_free (osso, service, object_path, interface, cb, data, NULL);
+    return osso_rpc_set_cb_f_with_free (osso, service, object_path,
+                                        interface, cb, data, NULL);
 }
 
 /************************************************************************/
@@ -629,26 +652,27 @@ osso_return_t osso_rpc_set_default_cb_f_with_free (osso_context_t *osso,
 						   gpointer data,
 						   osso_rpc_retval_free_f *retval_free)
 {
-    gchar interface[MAX_IF_LEN] = {0};
-    _osso_rpc_t *rpc;
+    _osso_callback_data_t *rpc;
     
-    if( (osso == NULL) || (cb == NULL) )
+    if (osso == NULL || cb == NULL)
 	return OSSO_INVALID;
 
-    rpc = (_osso_rpc_t *)calloc(1, sizeof(_osso_rpc_t));
+    rpc = calloc(1, sizeof(_osso_callback_data_t));
     if (rpc == NULL) {
         ULOG_ERR_F("calloc failed");
 	return OSSO_ERROR;
     }
     
-    g_snprintf(interface, MAX_IF_LEN, OSSO_BUS_ROOT ".%s",
-               osso->application);    
-    rpc->func = cb;
-    rpc->retval_free = retval_free;
-    rpc->data = data;
+    rpc->user_cb = cb;
+    rpc->user_data = data;
+    rpc->data = retval_free;
     
-    _msg_handler_set_cb_f_free_data(osso, interface, _rpc_handler,
-			  (gpointer)rpc, TRUE);
+    _msg_handler_set_cb_f_free_data(osso,
+                                    osso->service,
+                                    osso->object_path,
+                                    osso->interface,
+                                    _rpc_handler,
+                                    rpc, TRUE);
     return OSSO_OK;
 }
 
@@ -666,13 +690,21 @@ osso_return_t osso_rpc_unset_cb_f(osso_context_t *osso,
 				  const gchar *interface,
 				  osso_rpc_cb_f *cb, gpointer data)
 {
-    if( (osso == NULL) || (service == NULL) || (object_path == NULL) ||
-	(interface == NULL) || (cb == NULL))
+    _osso_callback_data_t user_data;
+
+    if (osso == NULL || service == NULL || object_path == NULL
+	|| interface == NULL || cb == NULL || osso->conn == NULL) {
+        ULOG_ERR_F("invalid parameters");
 	return OSSO_INVALID;
+    }
 
-    dbus_connection_unregister_object_path(osso->conn, object_path);
+    user_data.user_cb = cb;
+    user_data.user_data = data;
+    user_data.data = NULL;
 
-    _rm_cb_f(osso, interface, cb, data);
+    _msg_handler_rm_cb_f(osso, service, object_path, interface,
+                         (const _osso_handler_f*)_rpc_handler,
+                         &user_data, TRUE);
 
     return OSSO_OK;
 }
@@ -682,15 +714,21 @@ osso_return_t osso_rpc_unset_cb_f(osso_context_t *osso,
 osso_return_t osso_rpc_unset_default_cb_f(osso_context_t *osso,
 					  osso_rpc_cb_f *cb, gpointer data)
 {
-    gchar interface[MAX_IF_LEN] = {0};
-    
-    if(osso == NULL) return OSSO_INVALID;
-    if(cb == NULL) return OSSO_INVALID;
-    
-    g_snprintf(interface, MAX_IF_LEN, OSSO_BUS_ROOT ".%s",
-	       osso->application);
+    _osso_callback_data_t user_data;
 
-    _rm_cb_f(osso, interface, cb, data);
+    if (osso == NULL || cb == NULL) {
+        ULOG_ERR_F("invalid parameters");
+        return OSSO_INVALID;
+    }
+
+    user_data.user_cb = cb;
+    user_data.user_data = data;
+    user_data.data = NULL;
+    
+    _msg_handler_rm_cb_f(osso, osso->service, osso->object_path,
+                         osso->interface,
+                         (const _osso_handler_f*)_rpc_handler,
+                         &user_data, TRUE);
 
     return OSSO_OK;
 }
@@ -711,48 +749,20 @@ osso_return_t osso_rpc_set_timeout(osso_context_t *osso, gint timeout)
     osso->rpc_timeout = timeout;
     return OSSO_OK;
 }
-/************************************************************************/
-static void _rm_cb_f(osso_context_t *osso, const gchar *interface,
-		     osso_rpc_cb_f *cb, gpointer data)
-{   
-    guint i=0;
-    if( (osso == NULL) || (interface == NULL) || (cb == NULL) )
-	return;
-
-    for(i = 0; i < osso->ifs->len; i++) {
-	_osso_interface_t *intf;
-	intf = &g_array_index(osso->ifs, _osso_interface_t, i);
-	if( (intf->handler == &_rpc_handler) &&
-	    (strcmp(intf->interface, interface)==0) &&
-	    (intf->method) ) {
-	    _osso_rpc_t *rpc;
-	    rpc = (_osso_rpc_t *)intf->data;
-	    if( (rpc->func == cb) && (rpc->data == data) ) {
-		g_free(intf->interface);
-		g_array_remove_index_fast(osso->ifs, i);
-		free(rpc);
-	    }
-	    return;
-	}
-    }
-
-    return;
-}
 
 /************************************************************************/
 
 static DBusHandlerResult _rpc_handler(osso_context_t *osso, DBusMessage *msg,
-			       gpointer data)
+                                      _osso_callback_data_t *rpc)
 {
-    _osso_rpc_t *rpc;
     DBusMessageIter iter;
     GArray *arguments;
     osso_rpc_t retval;
     gint ret;
     int i;
+    osso_rpc_cb_f *handler;
+    osso_rpc_retval_free_f *retval_free;
 
-    rpc = (_osso_rpc_t *)data;
-    dprint("rpc->func = %p",rpc->func);
     arguments = g_array_new(FALSE, FALSE, sizeof(osso_rpc_t));
 
     if (dbus_message_iter_init(msg, &iter)) {
@@ -770,11 +780,15 @@ static DBusHandlerResult _rpc_handler(osso_context_t *osso, DBusMessage *msg,
 	         break;
         }
     }
-    dprint("calling handler at %p",rpc->func);
+
+    handler = rpc->user_cb;
+    retval_free = rpc->data;
+
+    dprint("calling handler at %p", handler);
     retval.type = DBUS_TYPE_INVALID;
-    ret = (rpc->func)(dbus_message_get_interface(msg),
-		      dbus_message_get_member(msg), arguments, rpc->data,
-		      &retval);
+    ret = (*handler)(dbus_message_get_interface(msg),
+        	     dbus_message_get_member(msg), arguments,
+                     rpc->user_data, &retval);
     if (retval.type == DBUS_TYPE_STRING) {
        dprint("handler returned string '%s'", retval.value.s);
     }
@@ -784,8 +798,8 @@ static DBusHandlerResult _rpc_handler(osso_context_t *osso, DBusMessage *msg,
     g_array_free(arguments, TRUE);
 
     if(ret == OSSO_INVALID) {
-	if (rpc->retval_free)
-	    rpc->retval_free (&retval);
+	if (retval_free != NULL)
+	    (*retval_free)(&retval);
 	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
     }
     
@@ -819,8 +833,8 @@ static DBusHandlerResult _rpc_handler(osso_context_t *osso, DBusMessage *msg,
 	    dbus_message_unref(reply);
 	}
     }
-    if (rpc->retval_free)
-        rpc->retval_free (&retval);
+    if (retval_free)
+        (*retval_free)(&retval);
     return DBUS_HANDLER_RESULT_HANDLED;
 }
 
@@ -972,9 +986,7 @@ static void _async_return_handler(DBusPendingCall *pending, void *data)
     ULOG_INFO_F("At msg return handler");
     msg = dbus_pending_call_steal_reply(pending);
     if(msg == NULL) {
-	g_free(rpc->interface);
-	g_free(rpc->method);
-	g_free(rpc);
+        free_osso_rpc_async_t(rpc);
 	return;
     }
     type = dbus_message_get_type(msg);
@@ -1012,10 +1024,7 @@ static void _async_return_handler(DBusPendingCall *pending, void *data)
     dprint("rpc->func = %p",rpc->data);
     dprint("rpc->interface = '%s'",rpc->interface);
     dprint("rpc->method = '%s'",rpc->method);
-    fflush(stderr);
-    g_free(rpc->interface);
-    g_free(rpc->method);
-    g_free(rpc);
+    free_osso_rpc_async_t(rpc);
     return;
 }
 
